@@ -2,7 +2,7 @@ package Importer;
 use strict qw/vars subs/; # Not refs!
 use warnings; no warnings 'once';
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 my %SIG_TO_SLOT = (
     '&' => 'CODE',
@@ -22,20 +22,18 @@ sub IMPORTER_MENU() {
         export_ok   => [qw/optimal_import/],
         export_anon => {
             import => sub {
-                my $class  = shift;
+                my $from  = shift;
                 my @caller = caller(0);
 
-                _version_check($class, \@caller, shift @_) if @_ && $NUMERIC{substr($_[0], 0, 1)};
+                _version_check($from, \@caller, shift @_) if @_ && $NUMERIC{substr($_[0], 0, 1)};
 
-                return unless @_;
-
-                my $file = _mod_to_file($class);
+                my $file = _mod_to_file($from);
                 _load_file(\@caller, $file) unless $INC{$file};
 
-                return if optimal_import($class, $caller[0], \@caller, @_);
+                return if optimal_import($from, $caller[0], \@caller, @_);
 
-                my $self = $class->new(
-                    from   => $class,
+                my $self = __PACKAGE__->new(
+                    from   => $from,
                     caller => \@caller,
                 );
 
@@ -98,7 +96,11 @@ sub import_into {
 
     my @caller;
 
-    if ($into =~ m/^\d+$/) {
+    if (ref($into)) {
+        @caller = @$into;
+        $into = $caller[0];
+    }
+    elsif ($into =~ m/^\d+$/) {
         @caller = caller($into + 1);
         $into = $caller[0];
     }
@@ -552,7 +554,7 @@ sub parse_args {
 
 sub _parse_args {
     my $self = shift;
-    my ($into, $menu, $args) = @_;
+    my ($into, $menu, $args, $is_tag) = @_;
 
     my $from = $self->from;
     my $main_menu = $self->menu($into);
@@ -576,7 +578,7 @@ sub _parse_args {
     my $set = pop @sets;
 
     $args = \@leftover;
-    @$args = (':DEFAULT') unless @$args || @versions;
+    @$args = (':DEFAULT') unless $is_tag || @$args || @versions;
 
     my %exclude;
     my @import;
@@ -642,7 +644,7 @@ sub _parse_args {
             substr($arg, 0, 1, '');
             my $tag = $menu->{tags}->{$arg} or $self->croak("$from does not export the :$arg tag");
 
-            my (undef, $cvers, $cexc, $cimp, $cset, $newmenu) = $self->_parse_args($into, $menu, $tag);
+            my (undef, $cvers, $cexc, $cimp, $cset, $newmenu) = $self->_parse_args($into, $menu, $tag, $arg);
 
             $self->croak("Exporter specified version numbers (" . join(', ', @$cvers) . ") in the :$arg tag!")
                 if @$cvers;
@@ -787,7 +789,7 @@ sub { *{"$into\\::\$_[0]"} = \$_[1] }
             next;
         }
 
-        $menu->{on_use}->($pin) if $menu->{on_use} && !$used->{$pin || ''}++;
+        $menu->{on_use}->($caller, $pin) if $menu->{on_use} && !$used->{$pin || ''}++;
 
         # Find the thing we are actually shoving in a new namespace
         my $ref = $menu->{exports}->{$symbol};
@@ -1384,13 +1386,13 @@ This function will be called the first time an importer selects a pin (or if
 symbols are imported with no pin).
 
     sub EXPORT_ON_USE {
-        my $pin = shift;
+        my ($caller, $pin) = @_;
 
-        if ($pin eq '<NO PIN SPECIFIED>') {
-            print "Importing without using a pin.\n";
+        if (!defined($pin)) {
+            print "Importing without using a pin at $caller->[1] line $caller->[2].\n";
         }
         else {
-            print "Switched to pin $pin.\n"
+            print "Switched to pin $pin at $caller->[1] line $caller->[2].\n"
         }
     }
 
@@ -1512,6 +1514,8 @@ B<IMPORTER_MENU() must be defined in your package, not a base class!>
             export_gen => \%EXPORT_GEN,        # Hash of builders, key is symbol
                                                # name, value is sub that generates
                                                # the symbol ref.
+
+            export_on_use => \&on_use,         # on-use callback 
         );
     }
 
